@@ -7,8 +7,8 @@ require Exporter;
 
 @ISA 		= qw(Exporter);
 @EXPORT 	= qw(queryfdb);
-@EXPORT_OK	= qw(querymacs queryports);
-$VERSION	= 0.56;
+@EXPORT_OK	= qw(querymacs queryports queryat);
+$VERSION	= 0.58;
 
 use Net::SNMP;
 
@@ -21,6 +21,46 @@ sub connect {
                                  Community => $cla{comm});
 }
 
+sub queryat {
+   my ($key, $newkey, %final);
+   &connect(@_);
+
+   my $ifoid = '1.3.6.1.2.1.3.1.1.1';
+   my $ifref = $session->get_table($ifoid);
+
+   if ($session->error) {
+      return {error => "true"};
+      exit 1;
+   }
+
+   my $physoid = '1.3.6.1.2.1.3.1.1.2';
+   my $physref = $session->get_table($physoid);
+   
+   if ($session->error) {
+      return {error => "true"};
+      exit 1;
+   }
+
+   my $addroid = '1.3.6.1.2.1.3.1.1.3';
+   my $addrref = $session->get_table($addroid);
+
+   if ($session->error) {
+      return {error => "true"};
+      exit 1;
+   }
+
+   foreach $key (keys %{$physref}) {
+      next if (length($physref->{$key}) < 14);
+      $physref->{$key} =~ s/0x//;
+      ($newkey = $key) =~ s/$physoid//;
+      $final{$physref->{$key}} = 
+         $addrref->{$addroid . $newkey} . "|" .
+         $ifref->{$ifoid . $newkey};
+   }
+
+   return \%final;
+}
+
 sub queryfdb {
    my ($key, $newkey, %port, %final);
    &connect(@_);
@@ -108,8 +148,8 @@ BridgeQuery - Perl extension for retrieving bridge tables.
 
 =head1 SYNOPSIS
 
-  use BridgeQuery;
-  use BridgeQuery qw(querymacs queryports);
+  use SNMP::BridgeQuery;
+  use SNMP::BridgeQuery qw(querymacs queryports queryat);
 
   $fdb = queryfdb(host => $address,
                   comm => $community);
@@ -120,12 +160,13 @@ BridgeQuery - Perl extension for retrieving bridge tables.
 
 =head1 DESCRIPTION
 
-BridgeQuery polls a device which respond to SNMP Bridge Table
+BridgeQuery polls a device which responds to SNMP Bridge Table
 queries and generates a hash reference with each polled MAC
 address as the key and the associated port as the value.  The
 specific MIBs that are polled are described in RFC1493.
 
 SNMP::BridgeQuery requires Net::SNMP in order to function.
+(Checked for during 'perl Makefile.PL')
 
 Devices can be switches, bridges, or most anything that responds
 as a OSI Layer 2 component.  Layer 3 devices do not generally
@@ -137,6 +178,15 @@ Two other functions (querymacs & queryports) can be explicitly
 exported.  They work the same way as queryfdb, but they return MAC
 addresses or ports (respectively) with the SNMP MIB as the hash key.
 
+A newly added function (queryat) can be used on layer 3 switches
+to poll the Address Translation Tables.  This is similar to the
+data returned by the 'queryfdb' function, but it returns the IP
+address of the device and the associated interface (separated by a 
+pipe '|') with the MAC address as the key.  On a layer 3 switch,
+this interface probably does not correspond to a physical port, 
+but more likely refers to a vlan ID.  Using this function on a 
+layer 2 device will generate a 'trapped' error.
+
 =head1 ACKNOLEDGEMENTS
 
 David M. Town - Author of Net::SNMP
@@ -151,7 +201,7 @@ perl(1), perldoc(1) Net::SNMP.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001 John D. Shearer.  All rights reverved.
+Copyright (c) 2001-2003 John D. Shearer.  All rights reverved.
 This program is free software; you may redistribute it
 and/or modify it under the same terms as Perl itself.
 
